require 'rack'
require 'rack/request'
require 'rack/ext/request'

module Rack
  class UsefulProcline
    VERSION = "0.0.3"
    
    class << self
      attr_accessor :config
      
      def configure
        @config ||= Config.new
        yield(config) if block_given?
        @config
      end
    end
    
    class Config
      attr_accessor :opts
      
      def initialize
        @opts = {
          request_uuid_key:       'action_dispatch.request_id', # Rails' default
          show_request_uuid:      true,
          show_request_method:    true,
          show_request_path:      true
        }
      end
    end
    
    attr_accessor :app, :config
    
    def initialize(app)
      @app = app
      @config = Rack::UsefulProcline.configure
    end
    
    # Public
    def call(env)
      @request = Request.new(env)
      
      prefix = $0.split(']')[0] + "]"
      procline = prefix.dup
      procline = procline_before_request(procline)
      
      set_procline(procline[0..200])
      
      status, headers, body = @app.call(env)
      
      procline = prefix.dup
      procline = procline_after_request(procline, status)
      set_procline(procline)
      
      [status, headers, body]
    end
    
    def set_procline(value)
      $0 = sanitize_utf8(value)
    end
    
    def sanitize_utf8(string)
      return string.force_encoding('utf-8') if string.valid_encoding?
      string.chars.select(&:valid_encoding?).join.force_encoding('utf-8')
    end
    
    def procline_before_request(procline)
      if @config.opts[:show_request_method] && @request
        procline.concat(" : #{@request.request_method}")
      end
      
      if @config.opts[:show_request_path] && @request
        procline.concat(" : #{@request.path_info}")
      end
      
      if @config.opts[:show_request_uuid]
        uuid = request_uuid || "No request uuid found"
        procline.concat(" : #{uuid}")
      end
      
      procline
    end
    
    def request_uuid
      if @request
        uuid = @request.uuid(@config.opts[:request_uuid_key])
        uuid[0..7] if uuid
      end
    end
    
    def procline_after_request(procline, status)
      procline.concat(" : last status: #{status}.")
      if @config.opts[:show_request_uuid]
        uuid = request_uuid || "No request uuid found"
        procline += " last request_uuid: #{uuid}."
      end
      procline += " Waiting for req."
    end
  end
end
